package sun.awt.image;

import java.awt.image.IndexColorModel;
import java.io.BufferedInputStream;
import java.util.zip.InflaterInputStream;
import java.util.zip.Inflater;
import java.util.GregorianCalendar;
import java.awt.Color;
import java.io.IOException;
import java.io.InputStream;
import java.awt.image.ColorModel;
import java.util.Hashtable;

public class PNGImageDecoder extends ImageDecoder
{
    private static final int GRAY = 0;
    private static final int PALETTE = 1;
    private static final int COLOR = 2;
    private static final int ALPHA = 4;
    private static final int bKGDChunk = 1649100612;
    private static final int cHRMChunk = 1665684045;
    private static final int gAMAChunk = 1732332865;
    private static final int hISTChunk = 1749635924;
    private static final int IDATChunk = 1229209940;
    private static final int IENDChunk = 1229278788;
    private static final int IHDRChunk = 1229472850;
    private static final int PLTEChunk = 1347179589;
    private static final int pHYsChunk = 1883789683;
    private static final int sBITChunk = 1933723988;
    private static final int tEXtChunk = 1950701684;
    private static final int tIMEChunk = 1950960965;
    private static final int tRNSChunk = 1951551059;
    private static final int zTXtChunk = 2052348020;
    private int width;
    private int height;
    private int bitDepth;
    private int colorType;
    private int compressionMethod;
    private int filterMethod;
    private int interlaceMethod;
    private int gamma;
    private Hashtable properties;
    private ColorModel cm;
    private byte[] red_map;
    private byte[] green_map;
    private byte[] blue_map;
    private byte[] alpha_map;
    private int transparentPixel;
    private byte[] transparentPixel_16;
    private static ColorModel[] greyModels;
    private static final byte[] startingRow;
    private static final byte[] startingCol;
    private static final byte[] rowIncrement;
    private static final byte[] colIncrement;
    private static final byte[] blockHeight;
    private static final byte[] blockWidth;
    int pos;
    int limit;
    int chunkStart;
    int chunkKey;
    int chunkLength;
    int chunkCRC;
    boolean seenEOF;
    private static final byte[] signature;
    PNGFilterInputStream inputStream;
    InputStream underlyingInputStream;
    byte[] inbuf;
    private static boolean checkCRC;
    private static final int[] crc_table;
    
    private void property(String key, Object value) {
        if (value == null) {
            return;
        }
        if (this.properties == null) {
            this.properties = new Hashtable();
        }
        this.properties.put(key, value);
        return;
    }
    
    private void property(String s, float value) {
        this.property(s, new Float(value));
    }
    
    private final void pngassert(boolean b) throws IOException {
        if (!b) {
            PNGImageDecoder.PNGException ex = new PNGImageDecoder.PNGException("Broken file");
            ex.printStackTrace();
            throw ex;
        }
        return;
    }
    
    protected boolean handleChunk(int n, byte[] array, int offset, int length) throws IOException {
        switch (n) {
            case 1649100612: {
                Color color = null;
                switch (this.colorType) {
                    case 2:
                    case 6: {
                        this.pngassert(length == 6);
                        color = new Color(array[offset] & 0xFF, array[offset + 2] & 0xFF, array[offset + 4] & 0xFF);
                        break;
                    }
                    case 3:
                    case 7: {
                        this.pngassert(length == 1);
                        int n2 = array[offset] & 0xFF;
                        this.pngassert(this.red_map != null && n2 < this.red_map.length);
                        color = new Color(this.red_map[n2] & 0xFF, this.green_map[n2] & 0xFF, this.blue_map[n2] & 0xFF);
                        break;
                    }
                    case 0:
                    case 4: {
                        this.pngassert(length == 2);
                        int n3 = array[offset] & 0xFF;
                        color = new Color(n3, n3, n3);
                        break;
                    }
                }
                if (color != null) {
                    this.property("background", color);
                    break;
                }
                break;
            }
            case 1665684045: {
                this.property("chromaticities", new PNGImageDecoder.Chromaticities(this.getInt(offset), this.getInt(offset + 4), this.getInt(offset + 8), this.getInt(offset + 12), this.getInt(offset + 16), this.getInt(offset + 20), this.getInt(offset + 24), this.getInt(offset + 28)));
                break;
            }
            case 1732332865: {
                if (length != 4) {
                    throw new PNGImageDecoder.PNGException("bogus gAMA");
                }
                this.gamma = this.getInt(offset);
                if (this.gamma != 100000) {
                    this.property("gamma", (float)this.gamma / 100000.0f);
                    break;
                }
            }
            case 1749635924: {
                break;
            }
            case 1229209940: {
                return false;
            }
            case 1229278788: {
                break;
            }
            case 1229472850: {
                if (length != 13 || (this.width = this.getInt(offset)) == 0 || (this.height = this.getInt(offset + 4)) == 0) {
                    throw new PNGImageDecoder.PNGException("bogus IHDR");
                }
                this.bitDepth = this.getByte(offset + 8);
                this.colorType = this.getByte(offset + 9);
                this.compressionMethod = this.getByte(offset + 10);
                this.filterMethod = this.getByte(offset + 11);
                this.interlaceMethod = this.getByte(offset + 12);
                break;
            }
            case 1347179589: {
                int n4 = length / 3;
                this.red_map = new byte[n4];
                this.green_map = new byte[n4];
                this.blue_map = new byte[n4];
                int i;
                for (i = 0, ; i < n4; ++i, offset += 3) {
                    this.red_map[i] = array[offset];
                    this.green_map[i] = array[offset + 1];
                    this.blue_map[i] = array[offset + 2];
                }
                break;
            }
            case 1883789683: {
                break;
            }
            case 1933723988: {
                break;
            }
            case 1950701684: {
                int length2;
                for (length2 = 0; length2 < length && array[offset + length2] != 0; ++length2) {}
                if (length2 < length) {
                    this.property(new String(array, offset, length2), new String(array, offset + length2 + 1, length - length2 - 1));
                    break;
                }
                break;
            }
            case 1950960965: {
                this.property("modtime", new GregorianCalendar(this.getShort(offset + 0), this.getByte(offset + 2) - 1, this.getByte(offset + 3), this.getByte(offset + 4), this.getByte(offset + 5), this.getByte(offset + 6)).getTime());
                break;
            }
            case 1951551059: {
                Label_1168: {
                    switch (this.colorType) {
                        case 3:
                        case 7: {
                            if (this.red_map != null) {
                                length = this.red_map.length;
                            }
                            System.arraycopy(array, offset, this.alpha_map = new byte[length], 0, (length < length) ? length : length);
                            while (--length >= length) {
                                this.alpha_map[length] = -1;
                            }
                            break Label_1168;
                        }
                        case 2:
                        case 6: {
                            this.pngassert(length == 6);
                            if (this.bitDepth == 16) {
                                this.transparentPixel_16 = new byte[6];
                                for (int j = 0; j < 6; ++j) {
                                    this.transparentPixel_16[j] = (byte)this.getByte(offset + j);
                                }
                                break Label_1168;
                            }
                            this.transparentPixel = ((this.getShort(offset + 0) & 0xFF) << 16 | (this.getShort(offset + 2) & 0xFF) << 8 | (this.getShort(offset + 4) & 0xFF));
                            break Label_1168;
                        }
                        case 0:
                        case 4: {
                            this.pngassert(length == 2);
                            int short1 = this.getShort(offset);
                            int n5 = 0xFF & ((this.bitDepth == 16) ? (short1 >> 8) : short1);
                            this.transparentPixel = (n5 << 16 | n5 << 8 | n5);
                            break Label_1168;
                        }
                    }
                }
            }
        }
        return true;
    }
    
    @Override
    public void produceImage() throws IOException, ImageFormatException {
        try {
            int i = 0;
            while (i < PNGImageDecoder.signature.length) {
                if ((PNGImageDecoder.signature[i] & 0xFF) != this.underlyingInputStream.read()) {
                    throw new PNGImageDecoder.PNGException("Chunk signature mismatch");
                }
                ++i;
            }
            BufferedInputStream bufferedInputStream = new BufferedInputStream(new InflaterInputStream(this.inputStream, new Inflater()));
            this.getData();
            byte[] array = null;
            int[] array2 = null;
            int width = this.width;
            int n;
            switch (this.bitDepth) {
                case 1: {
                    n = 0;
                    break;
                }
                case 2: {
                    n = 1;
                    break;
                }
                case 4: {
                    n = 2;
                    break;
                }
                case 8: {
                    n = 3;
                    break;
                }
                case 16: {
                    n = 4;
                    break;
                }
                default: {
                    throw new PNGImageDecoder.PNGException("invalid depth");
                }
            }
            int width2;
            if (this.interlaceMethod != 0) {
                width *= this.height;
                width2 = this.width;
            }
            else {
                width2 = 0;
            }
            int n2 = this.colorType | this.bitDepth << 3;
            int n3 = (1 << ((this.bitDepth >= 8) ? 8 : this.bitDepth)) - 1;
            switch (this.colorType) {
                case 3:
                case 7: {
                    if (this.red_map == null) {
                        throw new PNGImageDecoder.PNGException("palette expected");
                    }
                    if (this.alpha_map == null) {
                        this.cm = new IndexColorModel(this.bitDepth, this.red_map.length, this.red_map, this.green_map, this.blue_map);
                    }
                    else {
                        this.cm = new IndexColorModel(this.bitDepth, this.red_map.length, this.red_map, this.green_map, this.blue_map, this.alpha_map);
                    }
                    array = new byte[width];
                    break;
                }
                case 0: {
                    n = ((n >= 4) ? 3 : n);
                    ColorModel cm = PNGImageDecoder.greyModels[n];
                    this.cm = cm;
                    if (cm == null) {
                        int n4 = 1 << (1 << n);
                        byte[] array3 = new byte[n4];
                        for (int j = 0; j < n4; ++j) {
                            array3[j] = (byte)(255 * j / (n4 - 1));
                        }
                        if (this.transparentPixel == -1) {
                            this.cm = new IndexColorModel(this.bitDepth, array3.length, array3, array3, array3);
                        }
                        else {
                            this.cm = new IndexColorModel(this.bitDepth, array3.length, array3, array3, array3, this.transparentPixel & 0xFF);
                        }
                        PNGImageDecoder.greyModels[n] = this.cm;
                    }
                    array = new byte[width];
                    break;
                }
                case 2:
                case 4:
                case 6: {
                    this.cm = ColorModel.getRGBdefault();
                    array2 = new int[width];
                    break;
                }
                default: {
                    throw new PNGImageDecoder.PNGException("invalid color type");
                }
            }
            this.setDimensions(this.width, this.height);
            this.setColorModel(this.cm);
            this.setHints((this.interlaceMethod != 0) ? 6 : 30);
            this.headerComplete();
            int n5 = (((this.colorType & 0x1) != 0x0) ? 1 : ((((this.colorType & 0x2) != 0x0) ? 3 : 1) + (((this.colorType & 0x4) != 0x0) ? 1 : 0))) * this.bitDepth;
            int n6 = n5 + 7 >> 3;
            int n7;
            int n8;
            if (this.interlaceMethod == 0) {
                n7 = -1;
                n8 = 0;
            }
            else {
                n7 = 0;
                n8 = 7;
            }
            while (++n7 <= n8) {
                int k = PNGImageDecoder.startingRow[n7];
                int n9 = PNGImageDecoder.rowIncrement[n7];
                int n10 = PNGImageDecoder.colIncrement[n7];
                int n11 = PNGImageDecoder.blockWidth[n7];
                int n12 = PNGImageDecoder.blockHeight[n7];
                int l = PNGImageDecoder.startingCol[n7];
                int n13 = (this.width - l + (n10 - 1)) / n10 * n5 + 7 >> 3;
                if (n13 == 0) {
                    continue;
                }
                int n14 = (this.interlaceMethod == 0) ? (n9 * this.width) : 0;
                int n15 = width2 * k;
                int n16 = 1;
                byte[] b = new byte[n13];
                byte[] array4 = new byte[n13];
                while (k < this.height) {
                    int read = bufferedInputStream.read();
                    int off = 0;
                    while (off < n13) {
                        int read2 = bufferedInputStream.read(b, off, n13 - off);
                        if (read2 <= 0) {
                            throw new PNGImageDecoder.PNGException("missing data");
                        }
                        off += read2;
                    }
                    this.filterRow(b, (n16 != 0) ? null : array4, read, n13, n6);
                    int n17 = 0;
                    while (l < this.width) {
                        if (array2 != null) {
                            switch (n2) {
                                case 70: {
                                    array2[l + n15] = ((b[n17] & 0xFF) << 16 | (b[n17 + 1] & 0xFF) << 8 | (b[n17 + 2] & 0xFF) | (b[n17 + 3] & 0xFF) << 24);
                                    n17 += 4;
                                    break;
                                }
                                case 134: {
                                    array2[l + n15] = ((b[n17] & 0xFF) << 16 | (b[n17 + 2] & 0xFF) << 8 | (b[n17 + 4] & 0xFF) | (b[n17 + 6] & 0xFF) << 24);
                                    n17 += 8;
                                    break;
                                }
                                case 66: {
                                    int n18 = (b[n17] & 0xFF) << 16 | (b[n17 + 1] & 0xFF) << 8 | (b[n17 + 2] & 0xFF);
                                    if (n18 != this.transparentPixel) {
                                        n18 |= 0xFF000000;
                                    }
                                    array2[l + n15] = n18;
                                    n17 += 3;
                                    break;
                                }
                                case 130: {
                                    int n19 = (b[n17] & 0xFF) << 16 | (b[n17 + 2] & 0xFF) << 8 | (b[n17 + 4] & 0xFF);
                                    int n20 = this.transparentPixel_16 != null;
                                    for (int n21 = 0; n20 && n21 < 6; n20 &= ((b[n17 + n21] & 0xFF) == (this.transparentPixel_16[n21] & 0xFF)), ++n21) {}
                                    if (n20 == 0) {
                                        n19 |= 0xFF000000;
                                    }
                                    array2[l + n15] = n19;
                                    n17 += 6;
                                    break;
                                }
                                case 68: {
                                    int n22 = b[n17] & 0xFF;
                                    array2[l + n15] = (n22 << 16 | n22 << 8 | n22 | (b[n17 + 1] & 0xFF) << 24);
                                    n17 += 2;
                                    break;
                                }
                                case 132: {
                                    int n23 = b[n17] & 0xFF;
                                    array2[l + n15] = (n23 << 16 | n23 << 8 | n23 | (b[n17 + 2] & 0xFF) << 24);
                                    n17 += 4;
                                    break;
                                }
                                default: {
                                    throw new PNGImageDecoder.PNGException("illegal type/depth");
                                }
                            }
                        }
                        else {
                            switch (this.bitDepth) {
                                case 1: {
                                    array[l + n15] = (byte)(b[n17 >> 3] >> 7 - (n17 & 0x7) & 0x1);
                                    ++n17;
                                    break;
                                }
                                case 2: {
                                    array[l + n15] = (byte)(b[n17 >> 2] >> (3 - (n17 & 0x3)) * 2 & 0x3);
                                    ++n17;
                                    break;
                                }
                                case 4: {
                                    array[l + n15] = (byte)(b[n17 >> 1] >> (1 - (n17 & 0x1)) * 4 & 0xF);
                                    ++n17;
                                    break;
                                }
                                case 8: {
                                    array[l + n15] = b[n17++];
                                    break;
                                }
                                case 16: {
                                    array[l + n15] = b[n17];
                                    n17 += 2;
                                    break;
                                }
                                default: {
                                    throw new PNGImageDecoder.PNGException("illegal type/depth");
                                }
                            }
                        }
                        l += n10;
                    }
                    if (this.interlaceMethod == 0) {
                        if (array2 != null) {
                            this.sendPixels(0, k, this.width, 1, array2, 0, this.width);
                        }
                        else {
                            this.sendPixels(0, k, this.width, 1, array, 0, this.width);
                        }
                    }
                    k += n9;
                    n15 += n9 * width2;
                    b = (array4 = array4);
                    n16 = 0;
                }
                if (this.interlaceMethod != 0) {
                    if (array2 != null) {
                        this.sendPixels(0, 0, this.width, this.height, array2, 0, this.width);
                    }
                    else {
                        this.sendPixels(0, 0, this.width, this.height, array, 0, this.width);
                    }
                }
                else {
                    continue;
                }
            }
            this.imageComplete(3, true);
        }
        catch (IOException ex) {
            if (!this.aborted) {
                this.property("error", ex);
                this.imageComplete(3, true);
                throw ex;
            }
            try {
                this.close();
            }
            catch (Throwable t) {}
        }
        finally {
            try {
                this.close();
            }
            catch (Throwable t2) {}
        }
    }
    
    private boolean sendPixels(int n, int n2, int n3, int n4, int[] array, int n5, int n6) {
        if (this.setPixels(n, n2, n3, n4, this.cm, array, n5, n6) <= 0) {
            this.aborted = true;
        }
        return !this.aborted;
    }
    
    private boolean sendPixels(int n, int n2, int n3, int n4, byte[] array, int n5, int n6) {
        if (this.setPixels(n, n2, n3, n4, this.cm, array, n5, n6) <= 0) {
            this.aborted = true;
        }
        return !this.aborted;
    }
    
    private void filterRow(byte[] array, byte[] array2, int n, int n2, int n3) throws IOException {
        int i = 0;
        switch (n) {
            case 0: {
                break;
            }
            case 1: {
                for (int j = n3; j < n2; ++j) {
                    int n4 = j;
                    array[n4] += array[j - n3];
                }
                break;
            }
            case 2: {
                if (array2 != null) {
                    while (i < n2) {
                        int n5 = i;
                        array[n5] += array2[i];
                        ++i;
                    }
                    break;
                }
                break;
            }
            case 3: {
                if (array2 != null) {
                    while (i < n3) {
                        int n7 = i;
                        array[n7] += (0xFF & array2[i]) >> 1;
                        ++i;
                    }
                    while (i < n2) {
                        int n8 = i;
                        array[n8] += (array2[i] & 0xFF) + (array[i - n3] & 0xFF) >> 1;
                        ++i;
                    }
                    break;
                }
                for (int k = n3; k < n2; ++k) {
                    int n6 = k;
                    array[n6] += (array[k - n3] & 0xFF) >> 1;
                }
                break;
            }
            case 4: {
                if (array2 != null) {
                    while (i < n3) {
                        int n10 = i;
                        array[n10] += array2[i];
                        ++i;
                    }
                    while (i < n2) {
                        int n11 = array[i - n3] & 0xFF;
                        int n12 = array2[i] & 0xFF;
                        int n13 = array2[i - n3] & 0xFF;
                        int n14 = n11 + n12 - n13;
                        int n15 = (n14 > n11) ? (n14 - n11) : (n11 - n14);
                        int n16 = (n14 > n12) ? (n14 - n12) : (n12 - n14);
                        int n17 = (n14 > n13) ? (n14 - n13) : (n13 - n14);
                        int n18 = i;
                        array[n18] += ((n15 <= n16 && n15 <= n17) ? n11 : ((n16 <= n17) ? n12 : n13));
                        ++i;
                    }
                    break;
                }
                for (int l = n3; l < n2; ++l) {
                    int n9 = l;
                    array[n9] += array[l - n3];
                }
                break;
            }
            default: {
                throw new PNGImageDecoder.PNGException("Illegal filter");
            }
        }
    }
    
    public PNGImageDecoder(InputStreamImageSource inputStreamImageSource, InputStream inputStream) throws IOException {
        super(inputStreamImageSource, inputStream);
        this.gamma = 100000;
        this.transparentPixel = -1;
        this.transparentPixel_16 = null;
        this.inbuf = new byte[4096];
        this.inputStream = new PNGFilterInputStream(this, inputStream);
        this.underlyingInputStream = this.inputStream.underlyingInputStream;
    }
    
    private void fill() throws IOException {
        if (!this.seenEOF) {
            if (this.pos > 0 && this.pos < this.limit) {
                System.arraycopy(this.inbuf, this.pos, this.inbuf, 0, this.limit - this.pos);
                this.limit -= this.pos;
                this.pos = 0;
            }
            else if (this.pos >= this.limit) {
                this.pos = 0;
                this.limit = 0;
            }
            int length = this.inbuf.length;
            while (this.limit < length) {
                int read = this.underlyingInputStream.read(this.inbuf, this.limit, length - this.limit);
                if (read <= 0) {
                    this.seenEOF = true;
                    break;
                }
                this.limit += read;
            }
        }
    }
    
    private boolean need(int n) throws IOException {
        if (this.limit - this.pos >= n) {
            return true;
        }
        this.fill();
        if (this.limit - this.pos >= n) {
            return true;
        }
        if (this.seenEOF) {
            return false;
        }
        byte[] inbuf = new byte[n + 100];
        System.arraycopy(this.inbuf, this.pos, inbuf, 0, this.limit - this.pos);
        this.limit -= this.pos;
        this.pos = 0;
        this.inbuf = inbuf;
        this.fill();
        return this.limit - this.pos >= n;
    }
    
    private final int getInt(int n) {
        return (this.inbuf[n] & 0xFF) << 24 | (this.inbuf[n + 1] & 0xFF) << 16 | (this.inbuf[n + 2] & 0xFF) << 8 | (this.inbuf[n + 3] & 0xFF);
    }
    
    private final int getShort(int n) {
        return (short)((this.inbuf[n] & 0xFF) << 8 | (this.inbuf[n + 1] & 0xFF));
    }
    
    private final int getByte(int n) {
        return this.inbuf[n] & 0xFF;
    }
    
    private final boolean getChunk() throws IOException {
        this.chunkLength = 0;
        if (!this.need(8)) {
            return false;
        }
        this.chunkLength = this.getInt(this.pos);
        this.chunkKey = this.getInt(this.pos + 4);
        if (this.chunkLength < 0) {
            throw new PNGImageDecoder.PNGException("bogus length: " + this.chunkLength);
        }
        if (!this.need(this.chunkLength + 12)) {
            return false;
        }
        this.chunkCRC = this.getInt(this.pos + 8 + this.chunkLength);
        this.chunkStart = this.pos + 8;
        if (this.chunkCRC != crc(this.inbuf, this.pos + 4, this.chunkLength + 4) && PNGImageDecoder.checkCRC) {
            throw new PNGImageDecoder.PNGException("crc corruption");
        }
        this.pos += this.chunkLength + 12;
        return true;
    }
    
    private void readAll() throws IOException {
        while (this.getChunk()) {
            this.handleChunk(this.chunkKey, this.inbuf, this.chunkStart, this.chunkLength);
        }
    }
    
    boolean getData() throws IOException {
        while (this.chunkLength == 0 && this.getChunk()) {
            if (!this.handleChunk(this.chunkKey, this.inbuf, this.chunkStart, this.chunkLength)) {
                continue;
            }
            this.chunkLength = 0;
        }
        return this.chunkLength > 0;
    }
    
    public static boolean getCheckCRC() {
        return PNGImageDecoder.checkCRC;
    }
    
    public static void setCheckCRC(boolean checkCRC) {
        PNGImageDecoder.checkCRC = checkCRC;
    }
    
    protected void wrc(int b) {
        b &= 0xFF;
        if (b <= 32 || b > 122) {
            b = 63;
        }
        System.out.write(b);
    }
    
    protected void wrk(int n) {
        this.wrc(n >> 24);
        this.wrc(n >> 16);
        this.wrc(n >> 8);
        this.wrc(n);
    }
    
    public void print() {
        this.wrk(this.chunkKey);
        System.out.print(" " + this.chunkLength + "\n");
    }
    
    private static int update_crc(int n, byte[] array, int n2, int n3) {
        while (--n3 >= 0) {
            n = (PNGImageDecoder.crc_table[(n ^ array[n2++]) & 0xFF] ^ n >>> 8);
        }
        return n;
    }
    
    private static int crc(byte[] array, int n, int n2) {
        return ~update_crc(-1, array, n, n2);
    }
    
    static {
        PNGImageDecoder.greyModels = new ColorModel[4];
        startingRow = new byte[] { 0, 0, 0, 4, 0, 2, 0, 1 };
        startingCol = new byte[] { 0, 0, 4, 0, 2, 0, 1, 0 };
        rowIncrement = new byte[] { 1, 8, 8, 8, 4, 4, 2, 2 };
        colIncrement = new byte[] { 1, 8, 8, 4, 4, 2, 2, 1 };
        blockHeight = new byte[] { 1, 8, 8, 4, 4, 2, 2, 1 };
        blockWidth = new byte[] { 1, 8, 4, 4, 2, 2, 1, 1 };
        signature = new byte[] { -119, 80, 78, 71, 13, 10, 26, 10 };
        PNGImageDecoder.checkCRC = true;
        crc_table = new int[256];
        for (int i = 0; i < 256; PNGImageDecoder.crc_table[i] = i, ++i) {
            for (int j = 0; j < 8; ++j) {
                if ((i & 0x1) != 0x0) {
                    i = (0xEDB88320 ^ i >>> 1);
                }
                else {
                    i >>>= 1;
                }
            }
        }
    }
    
    public class PNGException extends IOException
    {
        PNGException(String message) {
            super(message);
        }
    }
    
    public static class Chromaticities
    {
        public float whiteX;
        public float whiteY;
        public float redX;
        public float redY;
        public float greenX;
        public float greenY;
        public float blueX;
        public float blueY;
        
        Chromaticities(int n, int n2, int n3, int n4, int n5, int n6, int n7, int n8) {
            this.whiteX = (float)n / 100000.0f;
            this.whiteY = (float)n2 / 100000.0f;
            this.redX = (float)n3 / 100000.0f;
            this.redY = (float)n4 / 100000.0f;
            this.greenX = (float)n5 / 100000.0f;
            this.greenY = (float)n6 / 100000.0f;
            this.blueX = (float)n7 / 100000.0f;
            this.blueY = (float)n8 / 100000.0f;
        }
        
        @Override
        public String toString() {
            return "Chromaticities(white=" + this.whiteX + "," + this.whiteY + ";red=" + this.redX + "," + this.redY + ";green=" + this.greenX + "," + this.greenY + ";blue=" + this.blueX + "," + this.blueY + ")";
        }
    }
}
